#########################################################################################
#                                                                                       #
#          Create MAI dBASE File from XML File: Create_MAI_dbf_from_XML.py              #
#                                                                                       #
#          author: Joyce Witebsky, University of Wisconsin, Milwaukee                   #
#          contact: witebsky@uwm.edu                                                    #
#                                                                                       #
#                                                                                       #
#          requirements: ArcGIS 10 or higher and Python 2.7                             #
#                                                                                       #
#########################################################################################

#Set up the magic formulas necessary to run script in ArcGIS:
import sys
import xml.etree.ElementTree as ET
import arcpy

arcpy.env.overwriteOutput = True
try:
    #Get input and output table names from tool input and change to work with Python:
    inFile = arcpy.GetParameterAsText(0)
    inFile = inFile.replace("\\","/")
    arcpy.AddMessage("Input File: "+inFile)
    
    outFileinput = arcpy.GetParameterAsText(1)
    outFileinput = outFileinput.replace("\\","/")
    arcpy.AddMessage("Output File: "+outFileinput)
    outFilearray = outFileinput.rsplit("/",1)
    
    #newTable = arcpy.CreateTable_management("C:/temp", "newMAIXML.dbf")
    arcpy.AddMessage("Made the split")
    arcpy.AddMessage(outFilearray[0]+ " "+outFilearray[1])

    newTable = arcpy.CreateTable_management(outFilearray[0], outFilearray[1])
    #Use "Describe" function to get ArcGIS info on newly created empty table:
    descTable = arcpy.Describe(newTable)

    arcpy.AddMessage("Preparing to create: " + descTable.path + "\\" + descTable.baseName + "." + descTable.extension)

    #Create string of new table path and full name:
    inTable = descTable.path + "\\" + descTable.baseName + "." + descTable.extension

    #Create array (list of lists) with attribute information from MPROP.doc: 
    newArray = [["TAXKEY","TEXT",10,"TAXKEY"],
    ["HSE_NBR","LONG",5,"HSE_NBR"], 
    ["SFX","TEXT",3,"SFX"],
    ["DIR","TEXT",1,"DIR"],
    ["STREET","TEXT",18,"STREET"],
    ["STTYPE","TEXT",2,"STTYPE"],
    ["UNIT_NBR","TEXT",5,"UNIT_NBR"],
    ["ZIP_CODE","TEXT",9,"ZIP_CODE"],
    ["LAND_USE","LONG",4,"LAND_USE"],
    ["RCD_NBR","LONG",6,"RCD_NBR"],
    ["UPD_DATE","LONG",6,"UPD_DATE"],
    ["WARD","LONG",3,"WARD"],
    ["MLERNT", "LONG",3,"MAIL_ERROR_COUNT"],
    ["MAILSTAT", "TEXT",1,"MAIL_STATUS"],
    ["RESCOMFL", "TEXT",1,"RES_COM_FLAG"]]
    longNamearray = []
    arcpy.AddMessage("Created newArray")
    #Add fields to dBASE table based on information in newArray:
    for fInfo in newArray:
        fName = fInfo[0]
        fFtype = fInfo[1]
        fLen = fInfo[2]
        fXMLName = fInfo[3]
        longNamearray.append(fXMLName)
        if fFtype == "TEXT":
            arcpy.AddField_management(inTable, fName, fFtype, "","",fLen)
        else:
            arcpy.AddField_management(inTable, fName, fFtype, fLen)
    arcpy.AddMessage("Added Fields")
    #longNameset = set(longNamearray)
    #Remove field automatically added during table creation:
    arcpy.DeleteField_management(inTable,["Field1"])
    arcpy.AddMessage("Set up fields.")
    # Create insert cursor for table
    rows = arcpy.InsertCursor(inTable)
    inFields = arcpy.ListFields(inTable)

    #gfile = open('c:/temp/mai_xml_11_3_2014.xml','r')

    arcpy. AddMessage("Opening input file.")

    gfile = open(inFile,'r')
    arcpy. AddMessage("Input file has been read.")
    llist = list(gfile)
    gfile.close()

    arcpy. AddMessage("Input file is ready for XML processing.")

    listlen = len(llist)
    firstpiece = llist[0]+llist[1]
    MAIstr = ""
    extraArray = []
    arcpy. AddMessage("Input file is being processed.  This will take a while (4-5 minutes in SARUP Lab)!  Get a cup of coffee.  Browse the internet....")
    for i in range(2, listlen):
        xmlpiece = llist[i].strip()
        #arcpy.AddMessage(xmlpiece)
        if xmlpiece == '<MAI>':
            MAIstr = xmlpiece
            #arcpy.AddMessage("In if area"+MAIstr+str(i))
        elif xmlpiece == '</MAI>':
            MAIstr = MAIstr + xmlpiece
            row = rows.newRow()
            #arcpy.AddMessage(MAIstr +str(i))
            #Treat MAIstr as an XML file and pull out children tags
            root = ET.fromstring(MAIstr)
            
            for child in root:
                #If child.tag is not in longNamearray or extraArray, put it in extraArray to use in problem log later:
                if longNamearray.count(child.tag) == 0:
                    if extraArray.count(child.tag) == 0:
                        extraArray.append(child.tag)
                    

                #Find tag name in newArray, pull out appropriate info, create row and insert into table:        
                for fStuff in newArray:
                    if fStuff[3] == child.tag:
                        val = child.text
                        #arcpy.AddMessage(child.tag + ": "+val)
                        if len(val) > fStuff[2]:
                            val = val[:fStuff[2]]
                        if fStuff[1] != "TEXT":
                            row.setValue(fStuff[0],int(val))
                        else:
                            row.setValue(fStuff[0],val)
            rows.insertRow(row)
                        
         
        else:
            MAIstr = MAIstr + xmlpiece

            #arcpy.AddMessage("In else area"+str(i))
    del row
    del rows
    arcpy.AddMessage("XML processing is finished.")
    if len(extraArray) > 0:
        arcpy.AddMessage("Number of XML attributes not added: " + str(len(extraArray)))
        for tagname in extraArray:
            arcpy.AddMessage("XML tag found, but not processed: "+tagname)
        

except:
    # Use Python's built-in dir function to check if row and cursor objects
    # exist. If they do exist, delete them.
    #
    arcpy.AddMessage("Something went wrong!")
    for fnam in longNamearray:
        arcpy.AddMessage(fnam)
    if 'row' in dir():
        del row
    if 'rows' in dir():
        del rows

