#########################################################################################
#                                                                                       #
#          Create MPROP dBASE File from XML File: Createdbf_from_XML.py                 #
#                                                                                       #
#          author: Joyce Witebsky, University of Wisconsin, Milwaukee                   #
#          contact: witebsky@uwm.edu                                                    #
#                                                                                       #
#                                                                                       #
#          requirements: ArcGIS 10 or higher and Python 2.7                             #
#                                                                                       #
#########################################################################################

#Set up the magic formulas necessary to run script in ArcGIS:
import sys
import xml.etree.ElementTree as ET
import arcpy

arcpy.env.overwriteOutput = True
try:
    #Get input and output table names from tool input and change to work with Python:
    inFile = arcpy.GetParameterAsText(0)
    #inFile = "C:\\Users\\witebsky\\Desktop\\TestMPROP4\\TestMPROP2\\MpropXML\\Mpropshortnoroot.xml"
    inFile = inFile.replace("\\","/")
    arcpy.AddMessage("Input File: "+inFile)
    
    outFileinput = arcpy.GetParameterAsText(1)
    #outFileinput = "C:\\Users\\witebsky\\Desktop\\TestMPROP4\\TestMPROP2\\MpropXML\\Mpropshortnoroot.dbf"
    outFileinput = outFileinput.replace("\\","/")
    arcpy.AddMessage("Output File: "+outFileinput)
    outFilearray = outFileinput.rsplit("/",1)
    
    #newTable = arcpy.CreateTable_management("C:/temp", "newMPROPXML.dbf")
    arcpy.AddMessage("Made the split")
    arcpy.AddMessage(outFilearray[0]+ " "+outFilearray[1])

    newTable = arcpy.CreateTable_management(outFilearray[0], outFilearray[1])
    #Use "Describe" function to get ArcGIS info on newly created empty table:
    descTable = arcpy.Describe(newTable)

    arcpy.AddMessage("Preparing to create: " + descTable.path + "\\" + descTable.baseName + "." + descTable.extension)

    #Create string of new table path and full name:
    inTable = descTable.path + "\\" + descTable.baseName + "." + descTable.extension

    #Create array (list of lists) with attribute information from MPROP.doc: 
    newArray = [["TAXKEY","TEXT",10,"TAXKEY"],
    ["SUBACCT","TEXT",1,"SUB_ACCT"],
    ["YRASSMT","TEXT",4,"YR_ASSMT"],
    ["CHKDIGIT","TEXT",1,"CHK_DIGIT"],
    ["TAXRATECD","TEXT",2,"TAX_RATE_CD"],
    ["PLATPAGE","TEXT",5,"PLAT_PAGE"],
    ["HOUSENRLO","LONG",5,"HOUSE_NR_LO"],
    ["HOUSENRHI","LONG",5,"HOUSE_NR_HI"],
    ["HOUSENRSFX","TEXT",3,"HOUSE_NR_SFX"],
    ["DIR","TEXT",1,"SDIR"],
    ["STREET","TEXT",18,"STREET"],
    ["STTYPE","TEXT",2,"STTYPE"],
    ["CACLASS","TEXT",1,"C_A_CLASS"],
    ["CASYMBOL","TEXT",1,"C_A_SYMBOL"],
    ["CALAND","LONG",9,"C_A_LAND"],
    ["CAIMPRV","LONG",9,"C_A_IMPRV"],
    ["CATOTAL","LONG",9,"C_A_TOTAL"],
    ["CAEXMTYPE","TEXT",3,"C_A_EXM_TYPE"],
    ["CAEXMLAND","LONG",9,"C_A_EXM_LAND"],
    ["CAEXMIMPRV","LONG",9,"C_A_EXM_IMPRV"],
    ["CAEXMTOTAL","LONG",9,"C_A_EXM_TOTAL"],
    ["PACLASS","TEXT",1,"P_A_CLASS"],
    ["PASYMBOL","TEXT",1,"P_A_SYMBOL"],
    ["PALAND","LONG",9,"P_A_LAND"],
    ["PAIMPRV","LONG",9,"P_A_IMPRV"],
    ["PATOTAL","LONG",9,"P_A_TOTAL"],
    ["PAEXMTYPE","TEXT",3,"P_A_EXM_TYPE"],
    ["PAEXMLAND","LONG",9,"P_A_EXM_LAND"],
    ["PAEXMIMPRV","LONG",9,"P_A_EXM_IMPRV"],
    ["PAEXMTOTAL","LONG",9,"P_A_EXM_TOTAL"],
    ["LASTVALUE","TEXT",21,"LAST_VALUE_CHG"],
    ["REASONFOR","TEXT",3,"REASON_FOR_CHG"],
    ["CONVEYDATE","TEXT",21,"CONVEY_DATE"],
    ["CONVEYTYPE","TEXT",2,"CONVEY_TYPE"],
    ["CONVEYFEE","FLOAT",7,"CONVEY_FEE"],
    ["CHGNR","TEXT",6,"CHG_NR"],
    ["DIVORG","SHORT",4,"DIV_ORG"],
    ["DIVDROP","SHORT",4,"DIV_DROP"],
    ["OWNERNAME1","TEXT",28,"OWNER_NAME_1"],
    ["OWNERNAME2","TEXT",28,"OWNER_NAME_2"],
    ["OWNERNAME3","TEXT",28,"OWNER_NAME_3"],
    ["OWNERADDR","TEXT",28,"OWNER_MAIL_ADDR"],
    ["OWNERCITY","TEXT",23,"OWNER_CITY_STATE"],
    ["OWNERZIP","TEXT",9,"OWNER_ZIP"],
    ["LASTNAMECH","TEXT",21,"LAST_NAME_CHG"],
    ["NEIGHBORHD","TEXT",4,"NEIGHBORHOOD"],
    ["EXMACREAGE","FLOAT",6,"EXM_ACREAGE"],
    ["PERCTLAND","FLOAT",5,"EXM_PER_CT_LAND"],
    ["PERCTIMPRV","FLOAT",5,"EXM_PER_CT_IMPRV"],
    ["BLDGTYPE","TEXT",9,"BLDG_TYPE"],
    ["NRSTORIES","FLOAT",4,"NR_STORIES"],
    ["BASEMENT","TEXT",1,"BASEMENT"],
    ["ATTIC","TEXT",1,"ATTIC"],
    ["NRUNITS","SHORT",3,"NR_UNITS"],
    ["BLDGAREA","LONG",7,"BLDG_AREA"],
    ["YRBUILT","TEXT",4,"YR_BUILT"],
    ["SWIMPOOL","TEXT",1,"SWIM_POOL"],
    ["FIREPLACE","TEXT",1,"FIREPLACE"],
    ["AIRCONDIT","TEXT",3,"AIR_CONDITIONING"],
    ["NRROOMS","TEXT",4,"NR_ROOMS"],
    ["BEDROOMS","SHORT",3,"BEDROOMS"],
    ["BATHS","SHORT",3,"BATHS"],
    ["POWDERROOM","SHORT",3,"POWDER_ROOMS"],
    ["NOFSPACES","SHORT",3,"NUMBER_OF_SPACES"],
    ["GARAGETYPE","TEXT",2,"PARKING_TYPE"],
    ["LOTAREA","LONG",9,"LOT_AREA"],
    ["CORNERLOT","TEXT",2,"CORNER_LOT"],
    ["ZONING","TEXT",7,"ZONING"],
    ["LANDUSE","TEXT",4,"LAND_USE"],
    ["LANDUSEGP","TEXT",2,"LAND_USE_GP"],
    ["OWNOCPD","TEXT",1,"OWN_OCPD"],
    ["GEOTRACT","TEXT",6,"GEO_TRACT"],
    ["GEOBLOCK","TEXT",4,"GEO_BLOCK"],
    ["GEOZIPCODE","TEXT",9,"GEO_ZIP_CODE"],
    ["GEOALDERO","TEXT",2,"GEO_ALDER_OLD"],
    ["GEOPOLICE","TEXT",2,"GEO_POLICE"],
    ["GEOFIRE","TEXT",2,"GEO_FIRE"],
    ["GEOBIMAINT","TEXT",2,"GEO_BI_MAINT"],
    ["DPWSANITAT","TEXT",2,"DPW_SANITATION"],
    ["GEOALDER","TEXT",2,"GEO_ALDER"],
    ["HISTCODE","TEXT",1,"HIST_CODE"],
    ["TAXDELQ","LONG",9,"TAX_DELQ"],
    ["BIVIOL","TEXT",4,"BI_VIOL"],
    ["RAZESTATUS","TEXT",1,"RAZE_STATUS"]]
    longNamearray = []
    #Add fields to dBASE table based on information in newArray:
    for fInfo in newArray:
        fName = fInfo[0]
        fFtype = fInfo[1]
        fLen = fInfo[2]
        fXMLName = fInfo[3]
        longNamearray.append(fXMLName)
        if fFtype == "TEXT":
            arcpy.AddField_management(inTable, fName, fFtype, "","",fLen)
        else:
            arcpy.AddField_management(inTable, fName, fFtype, fLen)
    #longNameset = set(longNamearray)
    #Remove field automatically added during table creation:
    arcpy.DeleteField_management(inTable, 
                                 ["Field1"])
    # Create insert cursor for table
    rows = arcpy.InsertCursor(inTable)
    inFields = arcpy.ListFields(inTable)

    #gfile = open('c:/temp/mprop_xml_11_3_2014.xml','r')

    arcpy. AddMessage("Opening input file.")

    gfile = open(inFile,'r')
    arcpy. AddMessage("Input file has been read.")
    newlist = list(gfile)
    gfile.close()
    
    arcpy. AddMessage("Input file is ready for XML processing.")

    listlen = len(newlist)
    if listlen == 1:
        newstring = str(newlist[0])
        newstring = newstring.replace("><",">\n<")
        llist = newstring.split("\n")
        #arcpy.AddMessage("Number in list: "+str(len(llist)))
        listlen = len(llist)
    else: llist = newlist

    MPROPstr = ""
    isMPROP = False
    extraArray = []
    arcpy. AddMessage("Input file is being processed.  This will take a while (10-15 minutes in SARUP Lab)!  Get a cup of coffee.  Browse the internet....")

    for i in range(0, listlen):
        xmlpiece = llist[i].strip()
        #arcpy.AddMessage(xmlpiece)
        if xmlpiece == '<MPROP>':
            MPROPstr = xmlpiece
            isMPROP = True
            #arcpy.AddMessage("In if area"+MPROPstr+str(i))
        elif xmlpiece == '</MPROP>':
            MPROPstr = MPROPstr + xmlpiece
            isMPROP = False
            #The end tag means the entire entry for the property has been captured and a new dbf row can be created:
            row = rows.newRow()
            #arcpy.AddMessage(MPROPstr +str(i))
            #Treat MPROPstr as an XML file and pull out children tags
            root = ET.fromstring(MPROPstr)
            
            for child in root:
                #If child.tag is not in longNamearray or extraArray, put it in extraArray to use in problem log later:
                if longNamearray.count(child.tag) == 0:
                    if extraArray.count(child.tag) == 0:
                        extraArray.append(child.tag)
                    

                #Find tag name in newArray, pull out appropriate info and add value:        
                for fStuff in newArray:
                    if (fStuff[3] == child.tag)and (child.text):
                        val = child.text
                        #arcpy.AddMessage(child.tag + ": " + val)
                        if len(val) > fStuff[2]:
                            val = val[:fStuff[2]]
                        if fStuff[1] != "TEXT":
                            row.setValue(fStuff[0],int(val))
                        else:
                            row.setValue(fStuff[0],val)
            #Insert row into table after all values in MPROPstr have been added:

            rows.insertRow(row)
            #arcpy.AddMessage("Inserted row."+str(i))            
         
        else:
            #If it's not the beginning of the string or the end, it must be somewhere in the middle so add it on:
            if isMPROP==True:
                MPROPstr = MPROPstr + xmlpiece
                #arcpy.AddMessage("In else area"+str(i))
    del row
    del rows
    arcpy.AddMessage("XML processing is finished.")
    if len(extraArray) > 0:
        arcpy.AddMessage("Number of XML attributes not added: " + str(len(extraArray)))
        for tagname in extraArray:
            arcpy.AddMessage("XML tag found, but not processed: "+tagname)
        

except:
    # Use Python's built-in dir function to check if row and cursor objects
    # exist. If they do exist, delete them.
    #
    if 'row' in dir():
        del row
    if 'rows' in dir():
        del rows
    arcpy.AddMessage("Something went wrong!")
    if len(extraArray) > 0:
        arcpy.AddMessage("Number of XML attributes not added: " + str(len(extraArray)))
    for tagname in extraArray:
        arcpy.AddMessage("XML tag found, but not processed: "+tagname)
        

