#########################################################################################
#                                                                                       #
#          Create MPROP Dbase File from ASCII File: CreatedbffromASCII.py               #
#                                                                                       #
#          author: Joyce Witebsky, University of Wisconsin, Milwaukee                   #
#          contact: witebsky@uwm.edu                                                    #
#                                                                                       #
#                                                                                       #
#          requirements: ArcGIS 10 or higher and Python 2.7                             #
#                                                                                       #
#########################################################################################

#Set up the magic formulas necessary to run script in ArcGIS:
import arcpy

arcpy.env.overwriteOutput = True

#Get output table name from tool input and change it to work with Python:

newTablename = arcpy.GetParameterAsText(1)
newTablename = newTablename.replace("\\","/")
arcpy.AddMessage("Output File: "+newTablename)

#Split up the output path and name to conform to the requirements of Create Table tool:
newTableArray = newTablename.rsplit("/",1)

newTable = arcpy.CreateTable_management(newTableArray[0], newTableArray[1])

#Use "Describe" function to get ArcGIS info on newly created empty table: 

descTable = arcpy.Describe(newTable)

arcpy.AddMessage("Path = " + descTable.path + " " + "File name = " +descTable.baseName + " " + "Extension = " +descTable.extension)

#Create string of new table path and full name:

inTable = descTable.path + "/" + descTable.baseName + "." + descTable.extension

#Create array (list of lists) with attribute information from MPROP.doc: 

newArray = [["TAXKEY","TEXT",10],
["SUBACCT","TEXT",1],
["YRASSMT","TEXT",4],
["CHKDIGIT","TEXT",1],
["TAXRATECD","TEXT",2],
["PLATPAGE","TEXT",5],
["HOUSENRLO","LONG",5],
["HOUSENRHI","LONG",5],
["HOUSENRSFX","TEXT",3],
["DIR","TEXT",1],
["STREET","TEXT",18],
["STTYPE","TEXT",2],
["CACLASS","TEXT",1],
["CASYMBOL","TEXT",1],
["CALAND","LONG",9],
["CAIMPRV","LONG",9],
["CATOTAL","LONG",9],
["CAEXMTYPE","TEXT",3],
["CAEXMLAND","LONG",9],
["CAEXMIMPRV","LONG",9],
["CAEXMTOTAL","LONG",9],
["PACLASS","TEXT",1],
["PASYMBOL","TEXT",1],
["PALAND","LONG",9],
["PAIMPRV","LONG",9],
["PATOTAL","LONG",9],
["PAEXMTYPE","TEXT",3],
["PAEXMLAND","LONG",9],
["PAEXMIMPRV","LONG",9],
["PAEXMTOTAL","LONG",9],
["LASTVALUE","TEXT",6],
["REASONFOR","TEXT",3],
["CONVEYDATE","TEXT",4],
["CONVEYTYPE","TEXT",2],
["CONVEYFEE","FLOAT",7],
["CHGNR","TEXT",6],
["DIVORG","SHORT",4],
["DIVDROP","SHORT",4],
["OWNERNAME1","TEXT",28],
["OWNERNAME2","TEXT",28],
["OWNERNAME3","TEXT",28],
["OWNERADDR","TEXT",28],
["OWNERCITY","TEXT",28],
["OWNERZIP","TEXT",9],
["LASTNAMECH","TEXT",6],
["NEIGHBORHD","TEXT",4],
["EXMACREAGE","FLOAT",6],
["PERCTLAND","FLOAT",5],
["PERCTIMPRV","FLOAT",5],
["BLDGTYPE","TEXT",9],
["NRSTORIES","FLOAT",4],
["BASEMENT","TEXT",1],
["ATTIC","TEXT",1],
["NRUNITS","SHORT",3],
["BLDGAREA","LONG",7],
["YRBUILT","TEXT",4],
["SWIMPOOL","TEXT",1],
["FIREPLACE","TEXT",1],
["AIRCONDIT","TEXT",3],
["NRROOMS","TEXT",4],
["BEDROOMS","SHORT",3],
["BATHS","SHORT",3],
["POWDERROOM","SHORT",3],
["NOFSPACES","SHORT",3],
["GARAGETYPE","TEXT",2],
["LOTAREA","LONG",9],
["CORNERLOT","TEXT",2],
["ZONING","TEXT",7],
["LANDUSE","TEXT",4],
["LANDUSEGP","TEXT",2],
["OWNOCPD","TEXT",1],
["GEOTRACT","TEXT",6],
["GEOBLOCK","TEXT",4],
["GEOZIPCODE","TEXT",9],
["GEOALDERO","TEXT",2],
["GEOPOLICE","TEXT",2],
["GEOFIRE","TEXT",2],
["GEOBIMAINT","TEXT",2],
["DPWSANITAT","TEXT",2],
["GEOALDER","TEXT",2],
["HISTCODE","TEXT",1],
["TAXDELQ","LONG",4],
["BIVIOL","TEXT",4],
["RAZESTATUS","TEXT",1]]

#Add MPROP fields to the new table and generate the number of total characters in one row (lineLen)
#based on attribute lengths from MPROP.doc:
lineLen = 0

for fInfo in newArray:
    fName = fInfo[0]
    fFtype = fInfo[1]
    fLen = fInfo[2]
    lineLen = lineLen + fLen
    if fFtype == "TEXT":
        arcpy.AddField_management(inTable, fName, fFtype, "","",fLen)
    else:
        arcpy.AddField_management(inTable, fName, fFtype, fLen)
#Delete the empty field added by the Create Table tool:        
arcpy.DeleteField_management(inTable,["Field1"])

#Get input file name from tool input and reformat to work with Python:
inFile = arcpy.GetParameterAsText(0)
inFile = inFile.replace("\\","/")
arcpy.AddMessage("Input File: "+inFile)
#Use Python commands to open input file for reading:
gfile = open(inFile,'r')
#Use Python list command to read input file:
llist = list(gfile)
#Now that it is in memory, close the input file: 
gfile.close()
#Get some idea of the size of the input:
listlen = len(llist)
arcpy.AddMessage("Line Length: " + str(lineLen))
#Get the first (and only) string in the file:
firststr  = llist[0]
arcpy.AddMessage("string length ="+str(len(firststr)))
#Create insert cursor called "rows" for new table:
rows = arcpy.InsertCursor(inTable)
#Create ArcGIS list of fields in new table:
inFields = arcpy.ListFields(inTable)
#Set up counters:
rowcount = 0
totalcount = 0
count = 0
try:
    #Limit looping to length of file:
    while totalcount < len(firststr):
        #Create a new row:
        row = rows.newRow()
        #Create a "slice" of the input file string containing the row attribute information: 
        valrow = firststr[totalcount:(totalcount + lineLen)]
        #Create a new array for values:
        valarray = []
        #Create a boolean to use to flag errors:
        isBad = False
        #Test for correct position of known constant values:
        if valrow[(count + lineLen - 5):(count + lineLen)] != "XXXX9":
            isBad = True
            #If known constant is not found, create a slice that compensates for known error:
            valrow = firststr[totalcount:(totalcount + lineLen - 3)]
        #Wend through the fields in the output table:
        for fld in inFields:
            #Skip OID field:
            if fld.name == "OID":
                continue
            else:
                #Add a message showing Taxkey if known error seems to be present and change slice length of known error string:
                if isBad and (fld.name == "GEOBLOCK"):
                    fLen2 = (fld.length - 3)
                    arcpy.AddMessage("Bad GEOBLOCK:"+valarray[0])
                else:
                    #Set a variable to contain attribute length for current field:
                    fLen2 = fld.length
                #Create a slice with the characters in the attribute in the current row:
                val = valrow[count:(count +fLen2)]
                #if rowcount == 155134:
                    #arcpy.AddMessage(fld.name + " " + fld.type + " " + str(fld.length) + " " + val)
                #All fields are strings or integers (though this may not have been the original intent):
                if fld.type != "String" and (val.isdigit()):
                    row.setValue(fld.name, int(val))
                else:
                    row.setValue(fld.name,val)
                #Add field names and values to list to use as output if there are unforeseen problems:
                valarray.append(fld.name + " " +val)
            #Add the current field length to the character count to use in next slice:
            count = count +fLen2
        #Add row to table when all attribute values have been added:
        rows.insertRow(row)
        #Need to increase count by 1 to avoid reusing last value:
        totalcount = totalcount + count + 1
        #Reset count:
        count = 0
        #Update row count:
        rowcount = rowcount + 1
        #if rowcount < 5:
            #arcpy.AddMessage("Totalcount ="+str(totalcount))

    # Delete cursor and row objects to remove locks on the data
    del row
    del rows
except:
    arcpy.AddMessage("Something went wrong here:")
    for valset in valarray:
        arcpy.AddMessage(valset)




    
    
